@@ -3,7 +3,7 @@
  * DUMA - Red-Zone memory allocator.
  *
  * Copyright (C) 2006 Michael Eddington <meddington@gmail.com>
- * Copyright (C) 2002-2007 Hayati Ayguen <h_ayguen@web.de>, Procitec GmbH
+ * Copyright (C) 2002-2008 Hayati Ayguen <h_ayguen@web.de>, Procitec GmbH
  * Copyright (C) 1987-1999 Bruce Perens <bruce@perens.com>
  *
  * License: GNU GPL (GNU General Public License, see COPYING-GPL)
@@ -732,8 +732,6 @@ void writeFile(const char * filename, unsigned long pagesize, int addrIdx, int s
   #define DUMA_EXTERN_C   extern
 #endif
 
-DUMA_EXTERN_C
-int get_new0strategy(void);
 
 
 int main()
@@ -741,7 +739,7 @@ int main()
   int iNumIntTypes, iIt;
   int addrIdx, sizeIdx;
   int alignment, max_sizeof;
-  int malloc0strategy, new0strategy;
+  int malloc0strategy;
   char  buffer[1024];
   char filename[1024];
 
@@ -810,10 +808,6 @@ int main()
       malloc0strategy = 3;
   }
 
-#if 0
-  new0strategy = get_new0strategy();
-#endif
-
 #ifdef _MSC_VER
   {
     /* fix output path when started form Visual C++ IDE */
