@@ -3,7 +3,7 @@
 #
 # Simple script to run DUMA on stuff.  (needs libduma.so).
 #
-# $Id: duma.sh,v 1.6 2008/03/09 11:45:21 h_ayguen Exp $
+# $Id: duma.sh,v 1.7 2008/07/20 22:59:32 h_ayguen Exp $
 #
 
 # Usage check.
@@ -14,4 +14,4 @@ if [ "$1" = "" ]; then
 fi
 
 # Set the environment to (pre)load duma library.
-(export LD_PRELOAD=libduma.so.0.0.0; export DYLD_INSERT_LIBRARIES=libduma.dylib; export DYLD_FORCE_FLAT_NAMESPACE=1; exec $*)
+(export LD_PRELOAD=./libduma.so.0.0.0; export DYLD_INSERT_LIBRARIES=libduma.dylib; export DYLD_FORCE_FLAT_NAMESPACE=1; exec $*)
